@@ -25,6 +25,7 @@
 #include <sstream> //For stringstream
 
 Document_Glom::Document_Glom()
+: m_block_cache_update(false)
 {
   //Conscious use of virtual methods in a constructor:
   set_file_extension("glom");
@@ -49,17 +50,17 @@ Document_Glom::~Document_Glom()
 {
 }
 
-Glib::ustring Document_Glom::get_connection_user()
+Glib::ustring Document_Glom::get_connection_user() const
 {
   return m_connection_user;
 }
 
-Glib::ustring Document_Glom::get_connection_server()
+Glib::ustring Document_Glom::get_connection_server() const
 {
   return m_connection_server;
 }
 
-Glib::ustring Document_Glom::get_connection_database()
+Glib::ustring Document_Glom::get_connection_database() const
 {
   return m_connection_database;
 }
@@ -118,6 +119,8 @@ void Document_Glom::set_relationship(const Glib::ustring& table_name, const Rela
       info.m_relationships.push_back(relationship);
     }
   }
+
+  update_cached_relationships();
 }
 
 bool Document_Glom::get_relationship(const Glib::ustring& table_name, const Glib::ustring& relationship_name, Relationship& relationship) const
@@ -243,7 +246,7 @@ Document_Glom::type_vecFields Document_Glom::get_table_fields(const Glib::ustrin
     }
     else
     {
-      g_warning("Document_Glom::get_table_fields: table not found in document: %s", table_name.c_str());
+      //g_warning("Document_Glom::get_table_fields: table not found in document: %s", table_name.c_str());
       return type_vecFields();
     }
   }
@@ -301,7 +304,7 @@ void Document_Glom::change_field_name(const Glib::ustring& table_name, const Gli
       //Change it:
       iterFind->set_name(strFieldNameNew);
     }
-    
+
     //Find any relationships or layouts that use this field
     //Look at each table:
     for(type_tables::iterator iter = m_tables.begin(); iter != m_tables.end(); ++iter)
@@ -334,9 +337,9 @@ void Document_Glom::change_field_name(const Glib::ustring& table_name, const Gli
     for(DocumentTableInfo::type_layouts::iterator iterLayouts = iterFindTable->second.m_layouts.begin(); iterLayouts != iterFindTable->second.m_layouts.end(); ++iterLayouts)
     {
       //Look at each group:
-      for(type_mapLayoutGroupSequence::iterator iterGroup = iterLayouts->second.begin(); iterGroup != iterLayouts->second.end(); ++iterGroup)
+      for(type_mapLayoutGroupSequence::iterator iterGroup = iterLayouts->m_layout_groups.begin(); iterGroup != iterLayouts->m_layout_groups.end(); ++iterGroup)
       {
-        //Change the field if it is in this group:    
+        //Change the field if it is in this group:
         iterGroup->second.change_field_item_name(strFieldNameOld, strFieldNameNew);
       }
     }
@@ -409,7 +412,7 @@ void Document_Glom::set_node_attribute_value_as_decimal(xmlpp::Element* node, co
   thestream.imbue( std::locale::classic() ); //The C locale.
   thestream << value;
   const Glib::ustring sequence_string = thestream.str();
-      
+
   set_node_attribute_value(node, strAttributeName, sequence_string);
 }
 
@@ -431,6 +434,28 @@ guint Document_Glom::get_node_attribute_value_as_decimal(const xmlpp::Element* n
   return result;
 }
 
+void Document_Glom::set_node_attribute_value_as_value(xmlpp::Element* node, const Glib::ustring& strAttributeName, const Gnome::Gda::Value& value,  Field::glom_field_type field_type)
+{
+  NumericFormat format_ignored; //Because we use ISO format.
+  const Glib::ustring value_as_text = GlomConversions::get_text_for_gda_value(field_type, value, std::locale() /* Use the C locale */, format_ignored, true /* ISO standard */);
+
+  set_node_attribute_value(node, strAttributeName, value_as_text);
+}
+
+Gnome::Gda::Value Document_Glom::get_node_attribute_value_as_value(const xmlpp::Element* node, const Glib::ustring& strAttributeName, Field::glom_field_type field_type)
+{
+  const Glib::ustring value_string = get_node_attribute_value(node, strAttributeName);
+
+  bool success = false;
+  Gnome::Gda::Value  result = GlomConversions::parse_value(field_type, value_string, success, true /* iso_format */);
+  if(success)
+    return result;
+  else 
+    return Gnome::Gda::Value();
+}
+
+
+
 Document_Glom::type_listTableInfo Document_Glom::get_tables() const
 {
   type_listTableInfo result;
@@ -514,7 +539,7 @@ Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_data_layout_groups
   if(child_table_name.empty())
     child_table_name = parent_table_name;
 
-  type_mapLayoutGroupSequence result = get_data_layout_groups(layout_name, parent_table_name);
+  type_mapLayoutGroupSequence result = get_data_layout_groups(layout_name, parent_table_name, child_table_name);
 
 
   //If there are no fields in the layout, then add a default:
@@ -525,7 +550,7 @@ Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_data_layout_groups
 
   if(create_default)
   {
-    g_warning("Document_Glom::get_data_layout_groups_plus_new_field(): Creating default layout for table %s, for layout %s", parent_table_name.c_str(), layout_name.c_str());
+    g_warning("Document_Glom::get_data_layout_groups_plus_new_fields(): Creating default layout for table %s (child_table=%s), for layout %s", parent_table_name.c_str(), child_table_name.c_str(), layout_name.c_str());
 
     //Get the last top-level group. We will add new fields into this one:
     //TODO_Performance: There must be a better way to do this:
@@ -582,18 +607,22 @@ Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_data_layout_groups
   return result;  
 }
   
-Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_data_layout_groups(const Glib::ustring& layout_name, const Glib::ustring& table_name) const
+Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_data_layout_groups(const Glib::ustring& layout_name, const Glib::ustring& parent_table_name, const Glib::ustring& table_name) const
 {
-  type_tables::const_iterator iterFind = m_tables.find(table_name);
+  Glib::ustring child_table = table_name;
+  if(child_table.empty())
+    child_table = parent_table_name;
+
+  type_tables::const_iterator iterFind = m_tables.find(parent_table_name);
   if(iterFind != m_tables.end())
   {
     const DocumentTableInfo& info = iterFind->second;
 
     //Look for the layout with this name:
-    DocumentTableInfo::type_layouts::const_iterator iter = info.m_layouts.find(layout_name);
+    DocumentTableInfo::type_layouts::const_iterator iter = std::find_if(info.m_layouts.begin(), info.m_layouts.end(), predicate_Layout<LayoutInfo>(child_table, layout_name));
     if(iter != info.m_layouts.end())
     {
-      return iter->second; //found   
+      return iter->m_layout_groups; //found
     }
   }
 
@@ -603,15 +632,33 @@ Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_data_layout_groups
 void Document_Glom::set_relationship_data_layout_groups(const Glib::ustring& layout_name, const Relationship& relationship, const type_mapLayoutGroupSequence& groups)
 {
   //TODO: Use an actual relationship_name instead of concatenating:
-  set_data_layout_groups(layout_name + "_related_" + relationship.get_name(), relationship.get_from_table(), groups);
+  set_data_layout_groups(layout_name + "_related_" + relationship.get_name(), relationship.get_from_table(), groups, relationship.get_to_table());
 }
 
-void Document_Glom::set_data_layout_groups(const Glib::ustring& layout_name, const Glib::ustring& table_name, const type_mapLayoutGroupSequence& groups)
+void Document_Glom::set_data_layout_groups(const Glib::ustring& layout_name, const Glib::ustring& parent_table_name, const type_mapLayoutGroupSequence& groups, const Glib::ustring& table_name)
 {
-  if(!table_name.empty())
+  Glib::ustring child_table_name = table_name;
+  if(child_table_name.empty())
+    child_table_name = parent_table_name;
+
+  //g_warning("Document_Glom::set_data_layout_groups(): ADDING layout for table %s (child_table=%s), for layout %s", parent_table_name.c_str(), child_table_name.c_str(), layout_name.c_str());
+
+
+  if(!parent_table_name.empty())
   {
-    DocumentTableInfo& info = get_table_info_with_add(table_name);
-    info.m_layouts[layout_name] = groups;
+    DocumentTableInfo& info = get_table_info_with_add(parent_table_name);
+
+    LayoutInfo layout_info;
+    layout_info.m_parent_table = child_table_name;
+    layout_info.m_layout_name = layout_name;
+    layout_info.m_layout_groups = groups;
+
+    DocumentTableInfo::type_layouts::iterator iter = std::find_if(info.m_layouts.begin(), info.m_layouts.end(), predicate_Layout<LayoutInfo>(child_table_name, layout_name));
+    if(iter == info.m_layouts.end())
+      info.m_layouts.push_back(layout_info);
+    else
+      *iter = layout_info;
+
     set_modified();
   }
 }
@@ -789,10 +836,7 @@ void Document_Glom::load_after_layout_group(const xmlpp::Element* node, const Gl
 
         item.set_name( get_node_attribute_value(element, "name") );
 
-        const Glib::ustring relationship_name = get_node_attribute_value(element, "relationship"); 
-        //item.set_relationship_name(relationship_name);
-        if(!relationship_name.empty())
-          get_relationship(table_name, relationship_name, item.m_relationship);
+        item.m_relationship.set_name( get_node_attribute_value(element, "relationship") ); //Full details are updated in filled-in ().
 
         item.set_editable( get_node_attribute_value_as_bool(element, "editable") );
 
@@ -805,6 +849,40 @@ void Document_Glom::load_after_layout_group(const xmlpp::Element* node, const Gl
 
         item.m_numeric_format = numeric_format;
 
+        //Choices:
+        item.set_choices_restricted( get_node_attribute_value_as_bool(element, "choices_restricted") );
+        item.set_has_custom_choices( get_node_attribute_value_as_bool(element, "choices_custom") );
+
+        if(item.get_has_custom_choices())
+        {
+          const xmlpp::Element* nodeChoiceList = get_node_child_named(element, "custom_choice_list");
+          if(nodeChoiceList)
+          {
+            LayoutItem_Field::type_list_values list_values;
+
+            xmlpp::Node::NodeList listNodesCustomChoices = nodeChoiceList->get_children("custom_choice");
+            for(xmlpp::Node::NodeList::iterator iter = listNodesCustomChoices.begin(); iter != listNodesCustomChoices.end(); ++iter)
+            {
+              const xmlpp::Element* element = dynamic_cast<const xmlpp::Element*>(*iter);
+              if(element)
+              {
+                Gnome::Gda::Value value = get_node_attribute_value_as_value(element, "value", item.m_field.get_glom_type());
+                list_values.push_back(value);
+              }
+            }
+
+            item.set_choices_custom(list_values);
+          }
+        }
+
+        item.set_has_related_choices( get_node_attribute_value_as_bool(element, "choices_related") );
+
+        item.set_choices(get_node_attribute_value(element, "choices_related_relationship"),
+          get_node_attribute_value(element, "choices_related_field"),
+          get_node_attribute_value(element, "choices_related_second") );
+        //Full details are updated in filled-in ().
+
+
         item.m_sequence = sequence;
         group.add_item(item, sequence);
       }
@@ -832,6 +910,8 @@ bool Document_Glom::load_after()
 {
   bool result = Bakery::Document_XML::load_after();  
 
+  m_block_cache_update = true; //Don't waste time repeatedly updating this until we have finished.
+
   if(result)
   {
     const xmlpp::Element* nodeRoot = get_node_document();
@@ -887,16 +967,15 @@ bool Document_Glom::load_after()
               {
                 Field field;
 
-                Gnome::Gda::FieldAttributes field_info = field.get_field_info();
                 const Glib::ustring strName = get_node_attribute_value(nodeChild, "name");
-                field_info.set_name( strName );
+                field.set_name( strName );
 
                 const Glib::ustring strTitle = get_node_attribute_value(nodeChild, "title");
                 field.set_title(strTitle);
  
-                field_info.set_primary_key( get_node_attribute_value_as_bool(nodeChild, "primary_key") );
-                field_info.set_unique_key( get_node_attribute_value_as_bool(nodeChild, "unique") );
-                field_info.set_auto_increment( get_node_attribute_value_as_bool(nodeChild, "auto_increment") );
+                field.set_primary_key( get_node_attribute_value_as_bool(nodeChild, "primary_key") );
+                field.set_unique_key( get_node_attribute_value_as_bool(nodeChild, "unique") );
+                field.set_auto_increment( get_node_attribute_value_as_bool(nodeChild, "auto_increment") );
 
                 //Get lookup information, if present.
                 xmlpp::Element* nodeLookup = get_node_child_named(nodeChild, "field_lookup");
@@ -922,12 +1001,7 @@ bool Document_Glom::load_after()
                   }
                 }
 
-                //Default value:
-                const Glib::ustring default_value_text = get_node_attribute_value(nodeChild, "default_value");
-                //Interpret the text as per the field type:
-                bool success = false;
-                field_info.set_default_value( GlomConversions::parse_value(field_type_enum, default_value_text, success, true /* iso_format */) );
-                field.set_field_info(field_info);
+                field.set_default_value( get_node_attribute_value_as_value(nodeChild, "default_value", field_type_enum) );
 
                 //We set this after set_field_info(), because that gets a glom type from the (not-specified) gdatype. Yes, that's strange, and should probably be more explicit.
                 field.set_glom_type( field_type_enum );
@@ -977,6 +1051,9 @@ bool Document_Glom::load_after()
               if(node)
               {
                 const Glib::ustring layout_name = get_node_attribute_value(node, "name");
+                Glib::ustring parent_table = get_node_attribute_value(node, "parent_table");
+                if(parent_table.empty())
+                  parent_table = table_name; //Deal with the earlier file format that did not include this.
 
                 type_mapLayoutGroupSequence layout_groups;
 
@@ -1002,7 +1079,11 @@ bool Document_Glom::load_after()
                   }
                 }
 
-                doctableinfo.m_layouts[layout_name] = layout_groups;
+                LayoutInfo layout_info;
+                layout_info.m_parent_table = parent_table;
+                layout_info.m_layout_name = layout_name;
+                layout_info.m_layout_groups = layout_groups;
+                doctableinfo.m_layouts.push_back(layout_info);
               }
             }
           } //if(nodeDataLayouts)
@@ -1053,6 +1134,9 @@ bool Document_Glom::load_after()
     }
   }
 
+  m_block_cache_update = false;
+  update_cached_relationships();
+
   return result;
 }
 
@@ -1093,6 +1177,30 @@ void Document_Glom::save_before_layout_group(xmlpp::Element* node, const LayoutG
         set_node_attribute_value_as_decimal(nodeItem, "format_decimal_places", field->m_numeric_format.m_decimal_places);
         set_node_attribute_value(nodeItem, "format_currency_symbol", field->m_numeric_format.m_currency_symbol);
 
+        set_node_attribute_value_as_bool(nodeItem, "choices_restricted", field->get_choices_restricted());
+        set_node_attribute_value_as_bool(nodeItem, "choices_custom", field->get_has_custom_choices());
+
+        if(field->get_has_custom_choices())
+        {
+           xmlpp::Element* child = nodeItem->add_child("custom_choice_list");
+
+           const LayoutItem_Field::type_list_values list_values = field->get_choices_custom();
+           for(LayoutItem_Field::type_list_values::const_iterator iter = list_values.begin(); iter != list_values.end(); ++iter)
+           {
+             xmlpp::Element* childChoice = child->add_child("custom_choice");
+             set_node_attribute_value_as_value(childChoice, "value", *iter, field->m_field.get_glom_type());
+           }
+        }
+
+        set_node_attribute_value_as_bool(nodeItem, "choices_related", field->get_has_related_choices() );
+
+        Glib::ustring choice_relationship, choice_field, choice_second;
+        field->get_choices(choice_relationship, choice_field, choice_second);
+        set_node_attribute_value(nodeItem, "choices_related_relationship", choice_relationship);
+        set_node_attribute_value(nodeItem, "choices_related_field", choice_field);
+        set_node_attribute_value(nodeItem, "choices_related_second", choice_second);
+
+
         set_node_attribute_value_as_decimal(nodeItem, "sequence", item->m_sequence);
       }
       else
@@ -1154,18 +1262,13 @@ bool Document_Glom::save_before()
           const Field& field = *iter;
 
           xmlpp::Element* elemField = elemFields->add_child("field");
-          set_node_attribute_value(elemField, "name", field.get_field_info().get_name());
+          set_node_attribute_value(elemField, "name", field.get_name());
           set_node_attribute_value(elemField, "title", field.get_title());
 
-          const Gnome::Gda::FieldAttributes field_info = field.get_field_info();
-          set_node_attribute_value_as_bool(elemField, "primary_key", field_info.get_primary_key());
-          set_node_attribute_value_as_bool(elemField, "unique", field_info.get_unique_key());
-          set_node_attribute_value_as_bool(elemField, "auto_increment", field_info.get_auto_increment());
-
-          NumericFormat format_ignored; //Because we use ISO format.
-          const Glib::ustring default_value_as_text = GlomConversions::get_text_for_gda_value(field.get_glom_type(), field_info.get_default_value(), std::locale() /* SQL uses the C locale */, format_ignored, true /* ISO standard */);
-          set_node_attribute_value(elemField, "default_value", default_value_as_text);
-
+          set_node_attribute_value_as_bool(elemField, "primary_key", field.get_primary_key());
+          set_node_attribute_value_as_bool(elemField, "unique", field.get_unique_key());
+          set_node_attribute_value_as_bool(elemField, "auto_increment", field.get_auto_increment());
+          set_node_attribute_value_as_value(elemField, "default_value", field.get_default_value(), field.get_glom_type());
 
           set_node_attribute_value(elemField, "calculation", field.get_calculation());
 
@@ -1212,11 +1315,12 @@ bool Document_Glom::save_before()
         for(DocumentTableInfo::type_layouts::const_iterator iter = doctableinfo.m_layouts.begin(); iter != doctableinfo.m_layouts.end(); ++iter)
         {
           xmlpp::Element* nodeLayout = nodeDataLayouts->add_child("data_layout");
-          nodeLayout->set_attribute("name", iter->first);
+          nodeLayout->set_attribute("name", iter->m_layout_name);
+          nodeLayout->set_attribute("parent_table", iter->m_parent_table);
 
           xmlpp::Element* nodeGroups = nodeLayout->add_child("data_layout_groups");
 
-          const type_mapLayoutGroupSequence& group_sequence = iter->second;
+          const type_mapLayoutGroupSequence& group_sequence = iter->m_layout_groups;
           for(type_mapLayoutGroupSequence::const_iterator iterGroups = group_sequence.begin(); iterGroups != group_sequence.end(); ++iterGroups)
           {
             save_before_layout_group(nodeGroups, iterGroups->second);
@@ -1329,3 +1433,75 @@ void Document_Glom::remove_group(const Glib::ustring& group_name)
     set_modified();
   }
 }
+
+void Document_Glom::update_cached_relationships(LayoutGroup& group, const Glib::ustring& table_name)
+{
+  //Find any LayoutItem_Fields, and fill in their full relationship details:
+  for(LayoutGroup::type_map_items::iterator iter = group.m_map_items.begin(); iter != group.m_map_items.end(); ++iter)
+  {
+    LayoutItem* pItem = iter->second;
+
+    LayoutItem_Field* pField = dynamic_cast<LayoutItem_Field*>(pItem);
+    if(pField)
+    {
+       if(pField->m_relationship.get_name_not_empty())
+       {
+         get_relationship(table_name, pField->m_relationship.get_name(), pField->m_relationship);
+       }
+
+       if(pField->m_choices_related_relationship.get_name_not_empty())
+       {
+         get_relationship(table_name, pField->m_choices_related_relationship.get_name(), pField->m_choices_related_relationship);
+       }
+    }
+    else
+    {
+      LayoutItem_Portal* pPortal = dynamic_cast<LayoutItem_Portal*>(pItem);
+      if(pPortal)
+      {
+
+        get_relationship(table_name, pPortal->get_relationship(), pPortal->m_relationship);
+
+        //Update the Portal's group: TODO: Do this in place, instead of using set/get.
+        const Glib::ustring layout_name = "list_related"; //TODO: This is silly.
+        type_mapLayoutGroupSequence portalGroups = get_relationship_data_layout_groups_plus_new_fields(layout_name, pPortal->m_relationship);
+
+        for(type_mapLayoutGroupSequence::iterator iterGroup = portalGroups.begin(); iterGroup != portalGroups.end(); ++iterGroup)
+        {
+          update_cached_relationships(iterGroup->second, pPortal->m_relationship.get_to_table());
+        }
+
+        set_relationship_data_layout_groups(layout_name, pPortal->m_relationship, portalGroups);
+      }
+      else
+      {
+        LayoutGroup* pGroup = dynamic_cast<LayoutGroup*>(pItem);
+        if(pGroup)
+          update_cached_relationships(*pGroup, table_name); //recurse:
+      }
+    }
+  }
+}
+
+void Document_Glom::update_cached_relationships()
+{
+  if(m_block_cache_update)
+    return;
+
+  //Update all the groups (groups are in layouts, in tables)
+  for(type_tables::iterator iterTable = m_tables.begin(); iterTable != m_tables.end(); ++iterTable)
+  {
+    DocumentTableInfo& tableInfo = iterTable->second;
+    for(DocumentTableInfo::type_layouts::iterator iterLayout = tableInfo.m_layouts.begin(); iterLayout != tableInfo.m_layouts.end(); ++iterLayout)
+    {
+      if(tableInfo.m_info.get_name() == iterLayout->m_parent_table) //If it is not a related layout:
+      {
+        type_mapLayoutGroupSequence& groups = iterLayout->m_layout_groups;
+        for(type_mapLayoutGroupSequence::iterator iterGroup = groups.begin(); iterGroup != groups.end(); ++iterGroup)
+        {
+          update_cached_relationships(iterGroup->second, tableInfo.m_info.get_name());
+        }
+      }
+    }
+  }
+}
