@@ -23,8 +23,12 @@
 #include "../data_structure/relationship.h"
 #include "../data_structure/glomconversions.h"
 #include "dialog_layout_details.h"
+#include <bakery/App/App_Gtk.h> //For util_bold_message().
+#include "../python_embed/glom_python.h"
+#include <libgnomevfsmm.h>
+#include <libgnome/gnome-url.h>
 #include <sstream> //For stringstream
-#include <libintl.h>
+#include <glibmm/i18n.h>
 
 Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
 : m_HBox(false, 6),
@@ -49,14 +53,15 @@ Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
     if(dialog)
     {
       m_pDialogLayout = dialog;
+      add_view(m_pDialogLayout); //Give it access to the document.
       m_pDialogLayout->signal_hide().connect( sigc::mem_fun(static_cast<Box_Data&>(*this), &Box_Data::on_dialog_layout_hide) );
     }
   }
-  
+
   m_FlowTable.set_columns_count(1); //Sub-groups will have multiple columns (by default, there is one sub-group, with 2 columns).
   m_FlowTable.set_padding(6);
-  
-  m_strHint = gettext("When you change the data in a field the database is updated immediately.\n Click [New] to add a new record.\n Leave automatic ID fields empty - they will be filled for you.");
+
+  m_strHint = _("When you change the data in a field the database is updated immediately.\n Click [New] to add a new record.\n Leave automatic ID fields empty - they will be filled for you.");
 
 
   //m_Paned.set_border_width(6);
@@ -66,7 +71,7 @@ Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
 
   //Related records:
   /*
-  m_Label_Related.set_text(gettext("<b>Related Records</b>"));
+  m_Label_Related.set_text(Bakery::App_Gtk::util_bold_message(_("Related Records")));
   m_Label_Related.set_use_markup(true);
   m_Frame_Related.set_label_widget(m_Label_Related);
   m_Frame_Related.set_shadow_type(Gtk::SHADOW_NONE);  
@@ -106,6 +111,9 @@ Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
   m_FlowTable.signal_field_edited().connect( sigc::mem_fun(*this,  &Box_Data_Details::on_flowtable_field_edited) );
   show_all();
 
+  m_FlowTable.signal_related_record_changed().connect( sigc::mem_fun(*this, &Box_Data_Details::on_flowtable_related_record_changed) );
+
+
   m_FlowTable.signal_layout_changed().connect( sigc::mem_fun(*this, &Box_Data_Details::on_flowtable_layout_changed) );
 
   m_ignore_signals = false;
@@ -113,6 +121,7 @@ Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
 
 Box_Data_Details::~Box_Data_Details()
 {
+  remove_view(&m_FlowTable);
 }
 
 Gnome::Gda::Value Box_Data_Details::get_primary_key_value() const
@@ -126,23 +135,26 @@ void Box_Data_Details::init_db_details(const Glib::ustring& strTableName, const
 
   get_field_primary_key_for_table(strTableName, m_field_primary_key);
 
-  Box_Data::init_db_details(strTableName);
+  Box_Data::init_db_details(strTableName); //Calls create_layout(), then fill_from_database()
 }
 
-void Box_Data_Details::refresh_db_details(const Gnome::Gda::Value& primary_key_value)
+void Box_Data_Details::refresh_data_from_database(const Gnome::Gda::Value& primary_key_value)
 {
-  init_db_details(get_table_name(), primary_key_value);
+  m_primary_key_value = primary_key_value;
+  fill_from_database();
 }
 
-void Box_Data_Details::refresh_db_details_blank()
+void Box_Data_Details::refresh_data_from_database_blank()
 {
-  refresh_db_details( Gnome::Gda::Value() );
+  refresh_data_from_database( Gnome::Gda::Value() );
 }
 
-void Box_Data_Details::fill_from_database_layout()
+void Box_Data_Details::create_layout()
 {
   Bakery::BusyCursor(*get_app_window());
 
+  Box_Data::create_layout(); //Fills m_TableFields.
+
   //Remove existing child widgets:
   m_FlowTable.remove_all();
 
@@ -153,6 +165,7 @@ void Box_Data_Details::fill_from_database_layout()
 
     //This map of layout groups will also contain the field information from the database:
     Document_Glom::type_mapLayoutGroupSequence layout_groups = get_data_layout_groups("details");
+
     for(Document_Glom::type_mapLayoutGroupSequence::const_iterator iter = layout_groups.begin(); iter != layout_groups.end(); ++iter)
     {
       m_FlowTable.add_layout_group(iter->second);
@@ -172,15 +185,13 @@ void Box_Data_Details::fill_from_database()
     sharedptr<SharedConnection> sharedconnection = connect_to_server();
 
 
-    Box_DB_Table::fill_from_database();
+    Box_Data::fill_from_database();
 
-    m_Fields = get_fields_to_show();
-    type_vecLayoutFields fieldsToGet = m_Fields;
+    m_FieldsShown = get_fields_to_show();
+    type_vecLayoutFields fieldsToGet = m_FieldsShown;
 
     if(!fieldsToGet.empty())
     {
-      fill_from_database_layout(); //TODO: Only do this when the layout has changed.
-
       //Do not try to show the data if the user may not view it:
       Privileges table_privs = get_current_privs(m_strTableName);
 
@@ -192,7 +203,6 @@ void Box_Data_Details::fill_from_database()
       {
         //Add extra possibly-non-visible columns that we need:
         LayoutItem_Field layout_item;
-        layout_item.set_name(m_field_primary_key.get_name());
         layout_item.m_field = m_field_primary_key;
         fieldsToGet.push_back(layout_item);
 
@@ -244,111 +254,23 @@ void Box_Data_Details::fill_from_database()
   }
 }
 
-/*
-void Box_Data_Details::fill_related()
-{
-  if(!m_bDoNotRefreshRelated)
-  {
-    //Clear existing pages:
-    {
-      Gtk::Notebook_Helpers::PageList& pages = m_Notebook_Related.pages();
-      for(Gtk::Notebook_Helpers::PageList::iterator iter = pages.begin(); iter != pages.end(); iter++)
-      {
-        Gtk::Notebook_Helpers::Page page = *iter;
-        Box_Data_List_Related* pWidget = dynamic_cast<Box_Data_List_Related*>(page.get_child());
-        if(pWidget)
-          remove_view(pWidget);
-        else
-          std::cout << "Box_Data_Details::fill_related(): unexpected child widget." << std::endl;
-      }
-
-      pages.clear(); //deletes Gtk::manage()d boxes.
-    }
-
-    //Get relationships from the document:
-    Document_Glom::type_vecRelationships vecRelationships = m_pDocument->get_relationships(m_strTableName);
-
-    if(vecRelationships.empty())
-    {
-      //Hide the relationships pane:
-      m_Frame_Related.hide();
-    }
-    else
-    {
-      m_Frame_Related.show();
-      
-      //Add the relationships:
-      for(Document_Glom::type_vecRelationships::iterator iter = vecRelationships.begin(); iter != vecRelationships.end(); iter++)
-      {
-       const Relationship& relationship = *iter;
-
-       //bool bMakeRelatedTab = true;
-       //guint rowKey = 0;
-
-       Glib::ustring from_field = relationship.get_from_field();
-       if(!from_field.empty())
-       {
-         const Glib::ustring to_table_name =  relationship.get_to_table();
-         const Glib::ustring to_field_name = relationship.get_to_field();
-
-         //Check that the to_field still exists in the to_table structure:
-         //(note that the to_field will not necessarily be visible in the related records list, though it must be in the database
-         //TODO_performance:
-         type_vecFields vecRelatedFields = get_fields_for_table_from_database(to_table_name); //Fields in the database;
-         type_vecFields::const_iterator iterFind = std::find_if(vecRelatedFields.begin(), vecRelatedFields.end(), predicate_FieldHasName<Field>(to_field_name));
-         
-         if(iterFind != vecRelatedFields.end() ) //If it was found
-         {
-           Box_Data_List_Related* pBox = Gtk::manage(new Box_Data_List_Related());
-           add_view(pBox); //So that it knows about the Document.
-           m_Notebook_Related.pages().push_back( Gtk::Notebook_Helpers::TabElem(*pBox, relationship.get_name()) );
-              
-           Gnome::Gda::Value value = m_FlowTable.get_field_value(from_field);
-
-           Field field;
-           bool test = get_fields_for_table_one_field(m_strTableName, from_field, field);
-           if(test)
-           {
-             pBox->init_db_details(relationship, value, m_primary_key_value);
-             pBox->show_all();
-
-
-             //Connect signals:
-             pBox->signal_record_added.connect( sigc::bind(
-               sigc::mem_fun(*this, &Box_Data_Details::on_related_record_added), relationship.get_from_field() )
-             );
-
-             pBox->signal_user_requested_details().connect( sigc::bind(
-               sigc::mem_fun(*this, &Box_Data_Details::on_related_user_requested_details), relationship.get_to_table() )
-             );
-           }
-         }
-       }
-      }
-    }
-  }
-
-}
-*/
-
 void Box_Data_Details::on_button_new()
 {
   if(confirm_discard_unstored_data())
   {
-    Gnome::Gda::FieldAttributes fieldinfo = m_field_primary_key.get_field_info();
-    if(fieldinfo.get_auto_increment()) //If the primary key is an auto-increment:
+    if(m_field_primary_key.get_auto_increment()) //If the primary key is an auto-increment:
     {
       //Just make a new record, and show it:
       Gnome::Gda::Value primary_key_value = generate_next_auto_increment(m_strTableName, m_field_primary_key.get_name()); //TODO: This should return a Gda::Value
 
       record_new(false /* use entered field data */, primary_key_value);
-      refresh_db_details(primary_key_value);
+      refresh_data_from_database(primary_key_value);
     }
     else
     {
       //It's not an auto-increment primary key,
       //so just blank the fields ready for a primary key later.
-      refresh_db_details_blank(); //shows blank record.
+      refresh_data_from_database_blank(); //shows blank record.
     }
 
   } //if(confirm_discard_unstored_data())
@@ -359,8 +281,8 @@ void Box_Data_Details::on_button_del()
   if( GlomConversions::value_is_empty(get_primary_key_value()) )
   {
     //Tell user that a primary key is needed to delete a record:
-    Gtk::MessageDialog dialog(gettext("<b>No primary key value.</b>"), true);
-    dialog.set_secondary_text(gettext("This record can not be deleted because there is no primary key."));
+    Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("No primary key value.")), true);
+    dialog.set_secondary_text(_("This record can not be deleted because there is no primary key."));
     dialog.set_transient_for(*get_app_window());
     dialog.run();
   }
@@ -432,6 +354,37 @@ void Box_Data_Details::on_related_user_requested_details(Gnome::Gda::Value key_v
 }
 */
 
+void Box_Data_Details::recalculate_fields_for_related_records(const Glib::ustring& relationship_name)
+{
+ //TODO: Get (recursively) the list of fields to recalcualte, sort them in order of dependency, then calculate them all.
+
+  const Gnome::Gda::Value primary_key_value = get_primary_key_value();
+
+  for(type_vecFields::iterator iter = m_TableFields.begin(); iter != m_TableFields.end(); ++iter)
+  {
+    const Field& field = *iter;
+
+    //Is this field triggered by this relationship?
+    const Field::type_list_strings triggered_by = field.get_calculation_relationships();
+    Field::type_list_strings::const_iterator iterFind = std::find(triggered_by.begin(), triggered_by.end(), relationship_name);
+    if(iterFind != triggered_by.end()) //If it was found
+    {
+       //recalculate:
+       const type_map_fields field_values = get_record_field_values(primary_key_value);
+       Gnome::Gda::Value value = glom_evaluate_python_function_implementation(field.get_glom_type(), field.get_calculation(), field_values,
+          get_document(), m_strTableName);
+
+      //show it, if it's on the layout:
+      LayoutItem_Field layout_item;
+      layout_item.m_field = field;
+      set_entered_field_data(layout_item, value);
+
+      //Add it to the database (even if it is not shown in the layout)
+      set_field_value_in_database(layout_item, value, m_field_primary_key, primary_key_value);
+    }
+  }
+}
+
 void Box_Data_Details::on_related_record_added(Gnome::Gda::Value /* strKeyValue */, Glib::ustring /* strFromKeyName */)
 {
   //Prevent deletion of Related boxes.
@@ -456,6 +409,8 @@ void Box_Data_Details::on_related_record_added(Gnome::Gda::Value /* strKeyValue
   }
   */
 
+  
+
   //Restore value:
   m_bDoNotRefreshRelated = bDoNotRefreshRelated;
 }
@@ -503,12 +458,17 @@ void Box_Data_Details::on_flowtable_layout_changed()
     document->set_data_layout_groups("details", m_strTableName, layout_groups);
 
   //Build the view again from the new layout:
-  fill_from_database_layout();
+  create_layout();
 
   //And fill it with data:
   fill_from_database();
 }
 
+void Box_Data_Details::on_flowtable_related_record_changed(const Glib::ustring& relationship_name)
+{
+  recalculate_fields_for_related_records(relationship_name);
+}
+
 void Box_Data_Details::on_flowtable_field_edited(const LayoutItem_Field& layout_field, const Gnome::Gda::Value& field_value)
 {
   if(m_ignore_signals)
@@ -569,15 +529,19 @@ void Box_Data_Details::on_flowtable_field_edited(const LayoutItem_Field& layout_
       }
     }
 
-    const Field& field = layout_field.m_field;
+    //const Field& field = layout_field.m_field;
+
+    //Set the value in all instances of this field in the layout (The field might be on the layout more than once):
+    m_FlowTable.set_field_value(layout_field, field_value);
 
     //Update the field in the record (the record with this primary key):
     try
     {
-      Glib::ustring strQuery = "UPDATE " + table_name;
-      strQuery += " SET " +  /* table_name + "." + postgres does not seem to like the table name here */ strFieldName + " = " + field.sql(field_value);
-      strQuery += " WHERE " + table_name + "." + primary_key_field.get_name() + " = " + primary_key_field.sql(primary_key_value);
-      bool bTest = Query_execute(strQuery);
+      bool bTest = set_field_value_in_database(layout_field, field_value, primary_key_field, primary_key_value);
+      //Glib::ustring strQuery = "UPDATE " + table_name;
+      //strQuery += " SET " +  /* table_name + "." + postgres does not seem to like the table name here */ strFieldName + " = " + field.sql(field_value);
+      //strQuery += " WHERE " + table_name + "." + primary_key_field.get_name() + " = " + primary_key_field.sql(primary_key_value);
+      //bool bTest = Query_execute(strQuery);
 
       if(!bTest)
       {
@@ -586,21 +550,12 @@ void Box_Data_Details::on_flowtable_field_edited(const LayoutItem_Field& layout_
       }
       else
       {
-        //Set the value in all instances of this field in the layout (The field might be on the layout more than once):
-        m_FlowTable.set_field_value(layout_field, field_value);
-
-        //Get-and-set values for lookup fields, if this field triggers those relationships:
-        do_lookups(layout_field, field_value, primary_key_field, primary_key_value);
-
-        //Show new values for related fields:
-        refresh_related_fields(layout_field, field_value, primary_key_field, primary_key_value);
-
         //TODO: Display new values for related fields.
 
         //If this is a foreign key then refresh the related records:
         /*
         bool bIsForeignKey = false;
-        Document_Glom::type_vecRelationships vecRelationships = m_pDocument->get_relationships(m_strTableName);
+        Document_Glom::type_vecRelationships vecRelationships = get_document()->get_relationships(m_strTableName);
         for(Document_Glom::type_vecRelationships::iterator iter = vecRelationships.begin(); iter != vecRelationships.end(); iter++)
         {
           const Relationship& relationship = *iter;
@@ -627,13 +582,13 @@ void Box_Data_Details::on_flowtable_field_edited(const LayoutItem_Field& layout_
   {
     //There is no current primary key value:
 
-    if(m_field_primary_key.get_field_info().get_auto_increment()) //If the primary key is an auto-increment:
+    if(m_field_primary_key.get_auto_increment()) //If the primary key is an auto-increment:
     {
       if(strFieldName == m_field_primary_key.get_name()) //If edited field is the primary key.
       {
         //Warn user that they can't choose their own primary key:
-        Gtk::MessageDialog dialog(gettext("<b>Primary key auto increments</b>"), true);
-        dialog.set_secondary_text(gettext("The primary key is auto-incremented.\n You may not enter your own primary key value."));
+        Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Primary key auto increments")), true);
+        dialog.set_secondary_text(_("The primary key is auto-incremented.\n You may not enter your own primary key value."));
         dialog.set_transient_for(*get_app_window());
         dialog.run();
       }
@@ -661,7 +616,7 @@ void Box_Data_Details::on_flowtable_field_edited(const LayoutItem_Field& layout_
   } //if(get_primary_key_value().size())
 }
 
-void Box_Data_Details::refresh_related_fields(const LayoutItem_Field& field_changed, const Gnome::Gda::Value& /* field_value */, const Field& primary_key, const Gnome::Gda::Value& primary_key_value)
+void Box_Data_Details::refresh_related_fields(const Gtk::TreeModel::iterator& /* row */, const LayoutItem_Field& field_changed, const Gnome::Gda::Value& /* field_value */, const Field& primary_key, const Gnome::Gda::Value& primary_key_value)
 {
   if(field_changed.get_has_relationship_name())
     return; //TODO: Handle these too.
@@ -703,7 +658,7 @@ void Box_Data_Details::refresh_related_fields(const LayoutItem_Field& field_chan
   }
 }
 
-void Box_Data_Details::do_lookups(const LayoutItem_Field& field_changed, const Gnome::Gda::Value& field_value, const Field& primary_key, const Gnome::Gda::Value& primary_key_value)
+void Box_Data_Details::do_lookups(const Gtk::TreeModel::iterator& /* row */, const LayoutItem_Field& field_changed, const Gnome::Gda::Value& field_value, const Field& primary_key, const Gnome::Gda::Value& primary_key_value)
 {
    if(field_changed.get_has_relationship_name())
     return; //TODO: Handle these too.
@@ -711,13 +666,13 @@ void Box_Data_Details::do_lookups(const LayoutItem_Field& field_changed, const G
    //Get values for lookup fields, if this field triggers those relationships:
    //TODO_performance: There is a LOT of iterating and copying here.
    const Glib::ustring strFieldName = field_changed.get_name();
-   type_list_lookups lookups = get_lookup_fields(strFieldName);
+   const type_list_lookups lookups = get_lookup_fields(strFieldName);
    for(type_list_lookups::const_iterator iter = lookups.begin(); iter != lookups.end(); ++iter)
    {
-     const LayoutItem_Field& layout_Item = iter->first;
+     const LayoutItem_Field& layout_item = iter->first;
 
      const Relationship relationship = iter->second;
-     const Field& field_lookup = layout_Item.m_field;
+     const Field& field_lookup = layout_item.m_field;
      const Glib::ustring field_lookup_name = field_lookup.get_name();
 
      Field field_source;
@@ -727,21 +682,13 @@ void Box_Data_Details::do_lookups(const LayoutItem_Field& field_changed, const G
        Gnome::Gda::Value value = get_lookup_value(iter->second /* relationship */,  field_source /* the field to look in to get the value */, field_value /* Value of to and from fields */);
 
        //Add it to the view:
-       LayoutItem_Field layout_item;
-       layout_item.set_name(field_lookup_name);
        //TODO? layout_item.set_relationship_name();
-       m_FlowTable.set_field_value(layout_item, value);
+       set_entered_field_data(layout_item, value);
 
        //Add it to the database (even if it is not shown in the view)
-       Glib::ustring strQuery = "UPDATE " + m_strTableName;
-       strQuery += " SET " + field_lookup_name + " = " + field_lookup.sql(value);
-       strQuery += " WHERE " + primary_key.get_name() + " = " + primary_key.sql(primary_key_value);
-       Query_execute(strQuery);  //TODO: Handle errors
-
-       //TODO: Handle lookups triggered by these fields (recursively)? TODO: Check for infinitely looping lookups.
+       set_field_value_in_database(layout_item, value, primary_key, primary_key_value);
      }
    }
-
 }
 
 void Box_Data_Details::on_userlevel_changed(AppState::userlevels user_level)
@@ -755,3 +702,136 @@ bool Box_Data_Details::get_field_primary_key(Field& field) const
   return false;
 }
 
+void Box_Data_Details::print_layout_group(xmlpp::Element* node_parent, const LayoutGroup& group)
+{
+  xmlpp::Element* nodeChildGroup = node_parent->add_child("group");
+  nodeChildGroup->set_attribute("title", group.m_title);
+
+  LayoutGroup::type_map_const_items items = group.get_items();
+  for(LayoutGroup::type_map_const_items::const_iterator iter = items.begin(); iter != items.end(); ++iter)
+  {
+    const LayoutItem* layout_item = iter->second;
+
+    const LayoutGroup* pLayoutGroup = dynamic_cast<const LayoutGroup*>(layout_item);
+    if(pLayoutGroup)
+      print_layout_group(nodeChildGroup, *pLayoutGroup); //recurse
+    else
+    {
+      const LayoutItem_Field* pLayoutField = dynamic_cast<const LayoutItem_Field*>(layout_item);
+      if(pLayoutField)
+      {
+        xmlpp::Element* nodeField = nodeChildGroup->add_child("field");
+
+        nodeField->set_attribute("title", pLayoutField->m_field.get_title_or_name());
+
+        Gnome::Gda::Value value = m_FlowTable.get_field_value(*pLayoutField);
+        Glib::ustring text_representation = GlomConversions::get_text_for_gda_value(pLayoutField->m_field.get_glom_type(), value,
+          pLayoutField->m_numeric_format); //In the current locale.
+
+        nodeField->set_attribute("value", text_representation);
+      }
+      else
+      {
+        const LayoutItem_Portal* pLayoutPortal = dynamic_cast<const LayoutItem_Portal*>(layout_item);
+        if(pLayoutPortal)
+        {
+          xmlpp::Element* nodePortal = nodeChildGroup->add_child("related_records");
+
+          //Box_Data_List_Related* pPortalWidget = m_FlowTable.get_portals();
+
+          Relationship relationship;
+          bool relationship_found = get_document()->get_relationship(m_strTableName, pLayoutPortal->get_relationship(), relationship);
+          if(relationship_found)
+          {
+            nodePortal->set_attribute("title", relationship.get_title_or_name());
+
+            //TODO:
+
+            //TODO: Only print this if the user has access rights.
+          }
+        }
+      }
+    }
+  }
+
+}
+
+void Box_Data_Details::print_layout()
+{
+  const Privileges table_privs = get_current_privs(m_strTableName);
+
+  //Don't try to print tables that the user can't view.
+  if(!table_privs.m_view)
+  {
+    //TODO: Warn the user.
+  }
+  else
+  {
+    //Create a DOM Document with the XML:
+    xmlpp::DomParser dom_parser;;
+
+    xmlpp::Document* pDocument = dom_parser.get_document();
+    xmlpp::Element* nodeRoot = pDocument->get_root_node();
+    if(!nodeRoot)
+    {
+      //Add it if it isn't there already:
+      nodeRoot = pDocument->create_root_node("details_print");
+    }
+
+    Glib::ustring table_title = get_document()->get_table_title(m_strTableName);
+    if(table_title.empty())
+      table_title = m_strTableName;
+
+    nodeRoot->set_attribute("table", table_title);
+
+
+    //The groups:
+    xmlpp::Element* nodeParent = nodeRoot;
+
+    Document_Glom::type_mapLayoutGroupSequence layout_groups = get_data_layout_groups("details");
+    for(Document_Glom::type_mapLayoutGroupSequence::const_iterator iter = layout_groups.begin(); iter != layout_groups.end(); ++iter)
+    {
+      const LayoutGroup& layout_group = iter->second;
+      print_layout_group(nodeParent, layout_group);
+    }
+
+
+    //Use libxslt to convert the XML to HTML:
+    Glib::ustring result = xslt_process(*pDocument, GLOM_XSLTDIR "print_details_to_html.xsl");
+    std::cout << "After xslt: " << result << std::endl;
+
+    //Save it to a temporary file and show it in a browser:
+    //TODO: This actually shows it in gedit.
+    const Glib::ustring temp_uri = "file:///tmp/glom_printout.html";
+    std::cout << "temp_uri=" << temp_uri << std::endl;
+
+    Gnome::Vfs::Handle write_handle;
+
+    try
+    {
+      //0660 means "this user and his group can read and write this non-executable file".
+      //The 0 prefix means that this is octal.
+      write_handle.create(temp_uri, Gnome::Vfs::OPEN_WRITE, false, 0660 /* leading zero means octal */);
+    }
+    catch(const Gnome::Vfs::exception& ex)
+    {
+      // If the operation was not successful, print the error and abort
+      return; // print_error(ex, output_uri_string);
+    }
+
+    try
+    {
+      //Write the data to the output uri
+      /* GnomeVFSFileSize bytes_written = */ write_handle.write(result.data(), result.bytes());
+    }
+    catch(const Gnome::Vfs::exception& ex)
+    {
+      // If the operation was not successful, print the error and abort
+      return; //print_error(ex, output_uri_string);
+    }
+
+    //Use the GNOME browser:
+    GError* error = 0;
+    gnome_url_show(temp_uri.c_str(), &error); //This is in libgnome.
+  }
+}
