@@ -50,8 +50,8 @@ public:
   virtual void on_Details_record_deleted(Gnome::Gda::Value primary_key_value);
 
 protected:
+  virtual void create_layout(); //override
   virtual void fill_from_database(); //override.
-  virtual void fill_column_titles();
 
   virtual bool get_field_primary_key_index(guint& field_column) const; //TODO: visible 
   virtual bool get_field_primary_key(Field& field) const;
@@ -63,7 +63,7 @@ protected:
   virtual void on_adddel_user_requested_add();
   virtual void on_adddel_user_requested_edit(const Gtk::TreeModel::iterator& row);
   virtual void on_adddel_user_requested_delete(const Gtk::TreeModel::iterator& rowStart, const Gtk::TreeModel::iterator& rowEnd);
-  virtual void on_adddel_user_added(const Gtk::TreeModel::iterator& row);
+  virtual void on_adddel_user_added(const Gtk::TreeModel::iterator& row, guint col_with_first_value);
   virtual void on_adddel_user_reordered_columns();
 
   virtual void on_adddel_user_requested_layout();
@@ -71,9 +71,12 @@ protected:
   virtual void on_adddel_user_changed(const Gtk::TreeModel::iterator& row, guint col);
 
   virtual void on_record_added(const Gnome::Gda::Value& primary_key_value); //Not a signal handler. To be overridden.
+  virtual void on_record_deleted(const Gnome::Gda::Value& primary_key_value);
 
   virtual bool get_field_column_index(const Glib::ustring& field_name, guint& index) const;
 
+  virtual void print_layout_group(xmlpp::Element* node_parent, const LayoutGroup& group);
+
   //Member widgers:
   mutable DbAddDel_WithButtons m_AddDel; //mutable because its get_ methods aren't const.
 
