@@ -20,12 +20,13 @@
 
 #include "db_adddel.h"
 #include <algorithm> //For std::find.
-#include <libintl.h>
-#include "../adddel/cellrendererlist.h"
+#include <glibmm/i18n.h>
+#include "../cellrendererlist.h"
 #include "db_treeviewcolumn_glom.h"
 #include "../../data_structure/glomconversions.h"
 #include "../../dialog_invalid_data.h"
 #include "../../application.h"
+//#include "../cellrendererlist.h"
 #include <iostream> //For debug output.
 
 DbAddDelColumnInfo::DbAddDelColumnInfo()
@@ -54,11 +55,12 @@ DbAddDelColumnInfo& DbAddDelColumnInfo::operator=(const DbAddDelColumnInfo& src)
 }
 
 DbAddDel::DbAddDel()
-: m_modelcolumn_key(0),
-  m_pMenuPopup(0),
+: m_pMenuPopup(0),
   m_auto_add(true),
   m_allow_add(true),
-  m_allow_delete(true)
+  m_allow_delete(true),
+  m_columns_ready(false),
+  m_allow_view(true)
 {
   set_prevent_user_signals();
   set_ignore_treeview_signals();
@@ -73,6 +75,7 @@ DbAddDel::DbAddDel()
 
   m_ScrolledWindow.add(m_TreeView);
   m_TreeView.show();
+  //m_TreeView.set_fixed_height_mode(); //This allows some optimizations.
   pack_start(m_ScrolledWindow);
 
   //Make sure that the TreeView doesn't start out only big enough for zero items.
@@ -95,24 +98,23 @@ DbAddDel::DbAddDel()
   set_ignore_treeview_signals(false);
 
   remove_all_columns(); //set up the default columns.
-  
+
   show_all_children();
 }
 
 DbAddDel::~DbAddDel()
 {
-  if(m_modelcolumn_key)
-    delete m_modelcolumn_key;
-  
+  //if(m_modelcolumn_key)
+  //  delete m_modelcolumn_key;
+
   //if(m_modelcolumn_placeholder)
   //  delete m_modelcolumn_placeholder;
-  
+
   App_Glom* pApp = get_application();
   if(pApp)
   {
     pApp->remove_developer_action(m_refContextLayout);
-  }
-   
+  } 
 }
 
 void
@@ -126,9 +128,10 @@ DbAddDel::on_MenuPopup_activate_Edit()
     {
       //Discover whether it's the last (empty) row:
       if(get_is_placeholder_row(iter))
-      {        
+      {
         //This is a new entry:
-        signal_user_added()(iter);
+        if(m_allow_add)
+          signal_user_added().emit(iter, 0);
 
         /*
         bool bRowAdded = true;
@@ -209,11 +212,12 @@ void DbAddDel::setup_menu()
   m_refContextAdd =  Gtk::Action::create("ContextAdd", Gtk::Stock::ADD);
   m_refActionGroup->add(m_refContextAdd,
     sigc::mem_fun(*this, &DbAddDel::on_MenuPopup_activate_Add) );
+  m_refContextAdd->set_sensitive(m_allow_add);
 
-  m_refContextLayout =  Gtk::Action::create("ContextLayout", gettext("Layout"));
+  m_refContextLayout =  Gtk::Action::create("ContextLayout", _("Layout"));
   m_refActionGroup->add(m_refContextLayout,
     sigc::mem_fun(*this, &DbAddDel::on_MenuPopup_activate_layout) );
-    
+
   //TODO: This does not work until this widget is in a container in the window:s
   App_Glom* pApp = get_application();
   if(pApp)
@@ -239,7 +243,7 @@ void DbAddDel::setup_menu()
         "    <menuitem action='ContextLayout'/>"        
         "  </popup>"
         "</ui>";
-        
+
     m_refUIManager->add_ui_from_string(ui_info);
   }
   catch(const Glib::Error& ex)
@@ -263,7 +267,7 @@ void DbAddDel::setup_menu()
     m_refContextEdit->set_sensitive(false);
     m_refContextDelete->set_sensitive(false);
   }
-  
+ 
   if(pApp)
     m_refContextLayout->set_sensitive(pApp->get_userlevel() == AppState::USERLEVEL_DEVELOPER);
 }
@@ -311,17 +315,20 @@ Gtk::TreeModel::iterator DbAddDel::get_item_placeholder()
     return add_item_placeholder();
   }
 }
-  
+
 Gtk::TreeModel::iterator DbAddDel::add_item_placeholder()
 {
-  Gtk::TreeModel::iterator iter = m_refListStore->append();
+  Gtk::TreeModel::iterator iter;
+
+  //Placeholder rows are for adding new records.
+  if(!m_allow_add)
+    return iter;
+
+  iter = m_refListStore->append();
   if(iter)
   {
-    Gtk::TreeModel::Row row = *iter;
-    row[*m_modelcolumn_key] = Gnome::Gda::Value(); //Remove temporary key value.
-    
     m_refListStore->set_is_placeholder(iter, true);
-    //row[*m_modelcolumn_placeholder] =  true; //This will be unset when set_value() is used to put real data in this column.
+    m_refListStore->set_key_value(iter, Gnome::Gda::Value()); //Remove temporary key value.
   }
 
   return iter;
@@ -340,6 +347,7 @@ Gtk::TreeModel::iterator DbAddDel::add_item(const Gnome::Gda::Value& valKey)
     if(treerow)
     {
       treerow[*m_modelcolumn_key] = valKey;
+      set_value_key(result, valKey);
       m_refListStore->set_is_placeholder(result, false);
       //treerow[*m_modelcolumn_placeholder] =  false; 
     }
@@ -354,6 +362,7 @@ void DbAddDel::remove_all()
 {
   InnerIgnore innerIgnore(this); //see comments for InnerIgnore class.
 
+/* TODO
   if(m_refListStore)
   {
     Gtk::TreeModel::iterator iter = m_refListStore->children().begin();
@@ -363,6 +372,7 @@ void DbAddDel::remove_all()
       iter = m_refListStore->children().begin();
     }
   }
+*/
 }
 
 
@@ -395,8 +405,7 @@ Gnome::Gda::Value DbAddDel::get_value_key_selected()
   Gtk::TreeModel::iterator iter = get_item_selected();
   if(iter)
   {
-    Gtk::TreeModel::Row row = *iter;
-    return row[*m_modelcolumn_key];
+    return get_value_key(iter);
   }
   else
     return Gnome::Gda::Value();
@@ -423,8 +432,8 @@ Gtk::TreeModel::iterator DbAddDel::get_row(const Gnome::Gda::Value& key)
 {
   for(Gtk::TreeModel::iterator iter = m_refListStore->children().begin(); iter != m_refListStore->children().end(); ++iter)
   {
-    Gtk::TreeModel::Row row = *iter;
-    const Gnome::Gda::Value& valTemp = row[*m_modelcolumn_key];
+    //Gtk::TreeModel::Row row = *iter;
+    const Gnome::Gda::Value& valTemp = get_value_key(iter);
     if(valTemp == key)
     {
       return iter;
@@ -505,10 +514,10 @@ void DbAddDel::add_blank()
 
   if(get_allow_user_actions()) //The extra blank line is only used if the user may add items:
   {
-    Gtk::TreeModel::iterator iter = get_last_row();        
+    Gtk::TreeModel::iterator iter = get_last_row();
     if(get_is_placeholder_row(iter))
     {
-        bAddNewBlank  = false; //One already exists.
+      bAddNewBlank  = false; //One already exists.
     }
     else
     {
@@ -517,7 +526,9 @@ void DbAddDel::add_blank()
   }
 
   if(bAddNewBlank)
+  {
     add_item_placeholder();
+  }
 
   set_prevent_user_signals(bPreventUserSignals);
 }
@@ -553,6 +564,8 @@ void DbAddDel::set_column_title(guint col, const Glib::ustring& strText)
 
 void DbAddDel::construct_specified_columns()
 {
+  InnerIgnore innerIgnore(this);
+
   //TODO_optimisation: This is called many times, just to simplify the API.
 
   //Delay actual use of set_column_*() stuff until this method is called.
@@ -571,15 +584,16 @@ void DbAddDel::construct_specified_columns()
   //Hidden internal columns:
 
   //We must recreate these standard modelcolumns, because we can not reuse them
-  if(m_modelcolumn_key)
-    delete m_modelcolumn_key; 
-  m_modelcolumn_key = new Gtk::TreeModelColumn<Gnome::Gda::Value>;
+  //if(m_modelcolumn_key)
+  //  delete m_modelcolumn_key;
+
+  //m_modelcolumn_key = new Gtk::TreeModelColumn<Gnome::Gda::Value>;
 
   //if(m_modelcolumn_placeholder)
   //  delete m_modelcolumn_placeholder;
   //m_modelcolumn_placeholder = new Gtk::TreeModelColumn<bool>;
 
-  record.add(*m_modelcolumn_key);
+  //record.add(*m_modelcolumn_key);
   //record.add(*m_modelcolumn_placeholder);
 
   //Database columns:
@@ -595,17 +609,45 @@ void DbAddDel::construct_specified_columns()
 
       record.add( *pModelColumn );
 
-      fields.push_back(iter->m_field.m_field);
+      fields.push_back(iter->m_field);
 
       i++;
     }
   }
 
+  //Find the primary key:
+  int column_index_key = 0;
+  bool key_found = false;
+  for(type_model_store::type_vec_fields::const_iterator iter = fields.begin(); iter != fields.end(); ++iter)
+  {
+    if( !(iter->get_has_relationship_name()) && (iter->m_field.get_primary_key()) )
+    {
+      key_found = true;
+      break;
+    }
+
+    ++column_index_key;
+  }
+
+  if(!key_found)
+  {
+    g_warning("DbAddDel::construct_specified_columns(): no primary key field found in:");
+    for(type_model_store::type_vec_fields::const_iterator iter = fields.begin(); iter != fields.end(); ++iter)
+    {
+      g_warning("  field: %s", iter->get_name().c_str());
+    }
+
+    g_assert(key_found); //crash here.
+  }
+
   //Create the model from the ColumnRecord:
-  m_refListStore = type_model_store::create(record, "TODO: table_name", fields);
+  m_refListStore = type_model_store::create(record, m_table_name, fields, column_index_key, m_allow_view, m_where_clause);
 
   m_TreeView.set_model(m_refListStore);
 
+
+
+
   //Remove all View columns:
   m_TreeView.remove_all_columns();
 
@@ -615,7 +657,7 @@ void DbAddDel::construct_specified_columns()
   for(type_vecModelColumns::iterator iter = vecModelColumns.begin(); iter != vecModelColumns.end(); ++iter)
   {
     DbAddDelColumnInfo& column_info = m_ColumnTypes[model_column_index];
-    if(column_info.m_visible)
+    if(column_info.m_visible && !(column_info.m_field.get_hidden())) //TODO: We shouldn't need both of these.
     {
       const Glib::ustring column_name = column_info.m_field.m_field.get_title_or_name();
       const Glib::ustring column_id = column_info.m_field.get_name();
@@ -636,14 +678,14 @@ void DbAddDel::construct_specified_columns()
           {
             pCellRendererDerived->append_list_item(*iter);
           }
-          
+
           pCellRenderer = pCellRendererDerived;
 
           break;
         }
-        
+
    */
-   
+
     /* Possible types:
     TYPE_INVALID,
     TYPE_NUMERIC,
@@ -652,7 +694,7 @@ void DbAddDel::construct_specified_columns()
     TYPE_TIME,
     TYPE_BOOLEAN
     */
-  
+
         case(Field::TYPE_BOOLEAN):
         {
           pCellRenderer = Gtk::manage( new Gtk::CellRendererToggle() );
@@ -661,7 +703,15 @@ void DbAddDel::construct_specified_columns()
         }
         default:
         {
-          pCellRenderer = Gtk::manage( new Gtk::CellRendererText() );
+          if(column_info.m_field.get_has_choices())
+          {
+            CellRendererList* rendererList = Gtk::manage( new CellRendererList() );
+            rendererList->set_restrict_values_to_list(column_info.m_field.get_choices_restricted());
+
+            pCellRenderer = rendererList;
+          }
+          else
+            pCellRenderer = Gtk::manage( new Gtk::CellRendererText() );
 
           break;
         }
@@ -677,15 +727,70 @@ void DbAddDel::construct_specified_columns()
         Gtk::CellRendererText* pCellRenderer = dynamic_cast<Gtk::CellRendererText*>(m_TreeView.get_column_cell_renderer(view_column_index));
         if(pCellRenderer)
         {
-          //Connect a signal handler:
-          if(pCellRenderer)
-          {
-            //Make it editable:
-            pCellRenderer->property_editable() = true;
+          //Make it editable:
+          pCellRenderer->property_editable() = true;
 
-            //Connect to its signal:
-            pCellRenderer->signal_edited().connect(
-              sigc::bind( sigc::mem_fun(*this, &DbAddDel::on_treeview_cell_edited), model_column_index) );
+          //Connect to its signal:
+          pCellRenderer->signal_edited().connect(
+            sigc::bind( sigc::mem_fun(*this, &DbAddDel::on_treeview_cell_edited), model_column_index) );
+
+          CellRendererList* pCellRendererCombo = dynamic_cast<CellRendererList*>(pCellRenderer);
+          if(pCellRendererCombo)
+          {
+            pCellRendererCombo->remove_all_list_items();
+
+            if(column_info.m_field.get_has_custom_choices())
+            {
+              pCellRendererCombo->set_use_second(false); //Custom choices have only one column.
+
+              //set_choices() needs this, for the numeric layout:
+              //pCellRendererCombo->set_layout_item(get_layout_item()->clone(), table_name); //TODO_Performance: We only need this for the numerical format.
+              const LayoutItem_Field::type_list_values list_values = column_info.m_field.get_choices_custom();
+              for(LayoutItem_Field::type_list_values::const_iterator iter = list_values.begin(); iter != list_values.end(); ++iter)
+              {
+                pCellRendererCombo->append_list_item( GlomConversions::get_text_for_gda_value(column_info.m_field.m_field.get_glom_type(), *iter, column_info.m_field.m_numeric_format) );
+              }
+            }
+            else if(column_info.m_field.get_has_related_choices())
+            {
+              Glib::ustring choice_relationship_name, choice_field, choice_second;
+              column_info.m_field.get_choices(choice_relationship_name, choice_field, choice_second);
+              if(!choice_relationship_name.empty() && !choice_field.empty())
+              {
+                const Relationship relationship = column_info.m_field.m_choices_related_relationship;
+                const Glib::ustring to_table = relationship.get_to_table();
+
+                const bool use_second = !choice_second.empty();
+                pCellRendererCombo->set_use_second(use_second);
+
+                LayoutItem_Field layout_field_second;
+                if(use_second)
+                {
+                  Document_Glom* document = get_document();
+                  if(document)
+                  {
+                    Field field_second; //TODO: Actually show this in the combo:
+                    document->get_field(to_table, choice_second, field_second);
+
+                    layout_field_second.m_field = field_second;
+
+                    //We use the default formatting for this field.
+                  }
+                }
+
+                type_list_values_with_second list_values = get_choice_values(column_info.m_field);
+                for(type_list_values_with_second::const_iterator iter = list_values.begin(); iter != list_values.end(); ++iter)
+                {
+                  const Glib::ustring first = GlomConversions::get_text_for_gda_value(column_info.m_field.m_field.get_glom_type(), iter->first, column_info.m_field.m_numeric_format);
+
+                  Glib::ustring second;
+                  if(use_second)
+                    second = GlomConversions::get_text_for_gda_value(layout_field_second.m_field.get_glom_type(), iter->second, layout_field_second.m_numeric_format);
+
+                  pCellRendererCombo->append_list_item(first, second);
+                }
+              }
+            }
           }
         }
         else
@@ -703,20 +808,24 @@ void DbAddDel::construct_specified_columns()
       }
 
     ++view_column_index;
-    
+ 
     } //is visible
 
     ++model_column_index;
   }
-  
+
   //Delete the vector's items:
   for(type_vecModelColumns::iterator iter = vecModelColumns.begin(); iter != vecModelColumns.end(); ++iter)
   {
      type_modelcolumn_value* pModelColumn = *iter;
-     delete pModelColumn;
+     if(pModelColumn)
+       delete pModelColumn;
   }
 
   m_TreeView.columns_autosize();
+
+  //Make sure there's a blank row after the database rows that have just been added.
+  add_blank();
 }
 
 void DbAddDel::set_value(const Gtk::TreeModel::iterator& iter, const LayoutItem_Field& layout_item, const Gnome::Gda::Value& value)
@@ -762,25 +871,35 @@ void DbAddDel::set_value_selected(const LayoutItem_Field& layout_item, const Gno
 void DbAddDel::remove_all_columns()
 {
   m_ColumnTypes.clear();
+
+  m_columns_ready = false;
 }
 
-guint DbAddDel::add_column(const LayoutItem_Field& field, bool editable, bool visible)
+void DbAddDel::set_table_name(const Glib::ustring& table_name)
+{
+  m_table_name = table_name;
+}
+
+guint DbAddDel::add_column(const LayoutItem_Field& field)
 {
   InnerIgnore innerIgnore(this); //Stop on_treeview_columns_changed() from doing anything when it is called just because we add a new column.
 
   DbAddDelColumnInfo column_info;
   column_info.m_field = field;
-  column_info.m_editable = editable;
-  column_info.m_visible = visible;
+  //column_info.m_editable = editable;
+  //column_info.m_visible = visible;
 
   //Make it non-editable if it is auto-generated:
-  if(field.m_field.get_field_info().get_auto_increment())
+  if(field.m_field.get_auto_increment())
     column_info.m_editable = false;
+  else
+    column_info.m_editable = field.get_editable_and_allowed();
 
   m_ColumnTypes.push_back(column_info);
 
   //Generate appropriate model columns:
-  construct_specified_columns();
+  if(m_columns_ready)
+    construct_specified_columns();
 
   //Tell the View to use the model:
   //m_TreeView.set_model(m_refListStore);
@@ -788,6 +907,17 @@ guint DbAddDel::add_column(const LayoutItem_Field& field, bool editable, bool vi
   return m_ColumnTypes.size() - 1;
 }
 
+void DbAddDel::set_where_clause(const Glib::ustring& where_clause)
+{
+  m_where_clause = where_clause;
+}
+
+void DbAddDel::set_columns_ready()
+{
+  m_columns_ready = true;
+  construct_specified_columns();
+}
+
 DbAddDel::type_list_indexes DbAddDel::get_column_index(const LayoutItem_Field& layout_item) const
 {
   //TODO_Performance: Replace all this looping by a cache/map:
@@ -852,7 +982,8 @@ void DbAddDel::set_column_choices(guint col, const type_vecStrings& vecStrings)
     else
     {
       //The column does not exist yet, so we must create it:
-      construct_specified_columns();
+      if(m_columns_ready)
+        construct_specified_columns();
     }
   }
 }
@@ -860,6 +991,7 @@ void DbAddDel::set_column_choices(guint col, const type_vecStrings& vecStrings)
 void DbAddDel::set_allow_add(bool val)
 {
   m_allow_add = val;
+  m_refContextAdd->set_sensitive(val);
 }
 
 void DbAddDel::set_allow_delete(bool val)
@@ -972,12 +1104,7 @@ Gnome::Gda::Value DbAddDel::treeview_get_key(const Gtk::TreeModel::iterator& row
 
   if(m_refListStore)
   {
-    Gtk::TreeModel::Row treerow = *row;
-
-    if(treerow)
-    {
-      return treerow[*m_modelcolumn_key];
-    }
+    return m_refListStore->get_key_value(row);
   }
 
   return value;
@@ -986,10 +1113,10 @@ Gnome::Gda::Value DbAddDel::treeview_get_key(const Gtk::TreeModel::iterator& row
 void DbAddDel::on_treeview_cell_edited_bool(const Glib::ustring& path_string, int model_column_index)
 {
   //Note:: model_column_index is actually the AddDel column index, not the TreeModel column index.
-  
+
   if(path_string.empty())
     return;
-    
+
   Gtk::TreePath path(path_string);
 
   //Get the row from the path:
@@ -999,7 +1126,7 @@ void DbAddDel::on_treeview_cell_edited_bool(const Glib::ustring& path_string, in
     Gtk::TreeModel::Row row = *iter;
 
     int tree_model_column_index = model_column_index + get_count_hidden_system_columns();
-    
+
     Gnome::Gda::Value value_old;
     row.get_value(tree_model_column_index, value_old);
 
@@ -1060,17 +1187,15 @@ void DbAddDel::on_treeview_cell_edited_bool(const Glib::ustring& path_string, in
 
       m_signal_user_changed.emit(row, model_column_index);
     }
-    
   }
 }
 
 void DbAddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const Glib::ustring& new_text, int model_column_index)
 {
   //Note:: model_column_index is actually the AddDel column index, not the TreeModel column index.
-  
   if(path_string.empty())
     return;
-  
+
   Gtk::TreePath path(path_string);
 
   //Get the row from the path:
@@ -1080,10 +1205,10 @@ void DbAddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const G
     Gtk::TreeModel::Row row = *iter;
 
     const int treemodel_column_index = model_column_index + get_count_hidden_system_columns();
-    
+
     Gnome::Gda::Value valOld;
     row.get_value(treemodel_column_index, valOld);
-    
+
     //Store the user's new text in the model:
     //row.set_value(treemodel_column_index, new_text);
 
@@ -1091,18 +1216,19 @@ void DbAddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const G
     bool bIsAdd = false;
     bool bIsChange = false;
     bool do_signal = true;
-    
+
     if(get_allow_user_actions()) //If add is possible:
     {
       if(get_is_placeholder_row(iter))
       {
         bool bPreventUserSignals = get_prevent_user_signals();
         set_prevent_user_signals(true); //Stops extra signal_user_changed.
-        
+
         //Mark this row as no longer a placeholder, because it has data now. The client code must set an actual key for this in the signal_user_added() or m_signal_user_changed signal handlers.
-        set_value_key(iter, Gnome::Gda::Value("glom_unknown"));
-        
-        add_item_placeholder(); //Add the next blank for the next user add.
+        //m_refListStore->set_is_placeholder(iter, false);
+        //Don't mark this as not a placeholder, because it's still a placeholder until it has a key value.
+
+        add_item_placeholder(); //Add the next blank for the next user add, if necessary.
         set_prevent_user_signals(bPreventUserSignals);
 
         bIsAdd = true; //Signal that a new key was added.
@@ -1114,11 +1240,11 @@ void DbAddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const G
     {
       //Make sure that the entered data is suitable for this field type:
       bool success = false;
-      Gnome::Gda::Value value = GlomConversions::parse_value(field_type, new_text, success);
+      Gnome::Gda::Value value = GlomConversions::parse_value(field_type, new_text, m_ColumnTypes[model_column_index].m_field.m_numeric_format, success);
       if(!success)
       {
           //Tell the user and offer to revert or try again:
-          bool revert = glom_show_dialog_invalid_date(field_type);
+          bool revert = glom_show_dialog_invalid_data(field_type);
           if(revert)
           {
             //Revert the data:
@@ -1132,7 +1258,7 @@ void DbAddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const G
             if(refTreeSelection)
             {
               refTreeSelection->select(row); //TODO: This does not seem to work.
-    
+
               if(!path.empty())
               {
                 Gtk::TreeView::Column* pColumn = m_TreeView.get_column(model_column_index);
@@ -1144,20 +1270,20 @@ void DbAddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const G
                     //TreeView::set_cursor(), or start_editing() would get the old value back from the model again
                     //so we do something similar without getting the old value:
                     m_TreeView.set_cursor(path, *pColumn, *pCell, true /* start_editing */); //This highlights the cell, and starts the editing.
-                    
+
                     //This is based on gtk_tree_view_start_editing():
                     //TODO: This does not actually work. I emailed gtk-list about how to do this.
                     /*
                     pCell->stop_editing();
                     pCell->property_text() = "test"; //new_text; //Allow the user to start with the bad text that he entered so far.
-                    
+
                     Gdk::Rectangle background_area;
                     m_TreeView.get_background_area(path, *pColumn, background_area);
-                                    
+
                     Gdk::Rectangle cell_area;
                     m_TreeView.get_cell_area(path, *pColumn, background_area); 
                     */
-                  }          
+                  }
                 }
               }
               else
@@ -1174,15 +1300,16 @@ void DbAddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const G
         //Store the value in the model:
         row.set_value(treemodel_column_index, value);
       }
-        
+
       if(!bIsAdd)
         bIsChange = true;
-  
+
       //Fire appropriate signal:
       if(bIsAdd)
       {
-        //Signal that a new key was added"        
-        m_signal_user_added.emit(row);
+        //Signal that a new key was added"
+        if(m_allow_add)
+          m_signal_user_added.emit(row, model_column_index);
       }
       else if(bIsChange)
       {
@@ -1386,17 +1513,26 @@ Glib::RefPtr<const Gtk::TreeModel> DbAddDel::get_model() const
 
 bool DbAddDel::get_is_first_row(const Gtk::TreeModel::iterator& iter) const
 {
-  return iter == get_model()->children().begin();
+  if(iter)
+    return iter == get_model()->children().begin();
+  else
+    return false;
 }
 
 bool DbAddDel::get_is_last_row(const Gtk::TreeModel::iterator& iter) const
 {
-  //TODO: Avoid this. iter::operator() might not work properly with our custom tree model.
-  return iter == get_last_row();
+  if(iter)
+  {
+    //TODO: Avoid this. iter::operator() might not work properly with our custom tree model.
+    return iter == get_last_row();
+  }
+  else
+    return false;
 }
 
 Gtk::TreeModel::iterator DbAddDel::get_next_available_row_with_add_if_necessary()
 {
+g_warning("get_next_available_row_with_add_if_necessary");
   Gtk::TreeModel::iterator result;
 
   if(!m_refListStore)
@@ -1440,43 +1576,12 @@ Gtk::TreeModel::iterator DbAddDel::get_next_available_row_with_add_if_necessary(
 
 Gtk::TreeModel::iterator DbAddDel::get_last_row() const
 {
-  Glib::RefPtr<const Gtk::TreeModel> refModel = get_model();
-  if(!refModel)
-    return Gtk::TreeModel::iterator();
-  else
-  {
-    //TODO_performance: Hopefully there is a better way to do this.
-    Gtk::TreeModel::iterator iter = get_model()->children().begin();
-    guint size = get_model()->children().size();
-    if(size > 1)
-    {
-      guint i = 0;
-      for(i = 0; i < (size -1); ++i)
-      {
-        ++iter;
-      }
-      
-      //g_warning("DbAddDel::get_last_row(): size=%d, i=%d", size, i);
-    }
-    
-    return iter;
-  }
+  return m_refListStore->get_last_row();
 }
 
 Gtk::TreeModel::iterator DbAddDel::get_last_row()
 {
-  //TODO_performance: Hopefully there is a better way to do this.
-  Gtk::TreeModel::iterator iter = get_model()->children().begin();
-  guint size = get_model()->children().size();
-  if(size > 1)
-  {
-    for(guint i = 0; i < (size -1); ++i)
-    {
-      ++iter;
-    }
-  }
-
-  return iter;
+  return m_refListStore->get_last_row();
 }
 
 Gnome::Gda::Value DbAddDel::get_value_key(const Gtk::TreeModel::iterator& iter)
@@ -1498,7 +1603,9 @@ void DbAddDel::set_value_key(const Gtk::TreeModel::iterator& iter, const Gnome::
       //row[*m_modelcolumn_placeholder] = false;
     }
 
-    row[*m_modelcolumn_key] = value;
+    m_refListStore->set_key_value(iter, value);
+
+   // row[*m_modelcolumn_key] = value;
   }
 }
 
@@ -1511,6 +1618,9 @@ bool DbAddDel::get_is_placeholder_row(const Gtk::TreeModel::iterator& iter) cons
     return false;
   }
 
+  if(!iter)
+    return false;
+
   if(iter == m_refListStore->children().end())
   {
     return false;
@@ -1547,7 +1657,8 @@ bool DbAddDel::get_view_column_index(guint model_column_index, guint& view_colum
 
 guint DbAddDel::get_count_hidden_system_columns()
 {
-  return 1; //The key.
+  return 0; //The key now has explicit API in the model.
+  //return 1; //The key.
   //return 2; //The key and the placeholder boolean.
 }
 
@@ -1568,30 +1679,35 @@ void DbAddDel::set_key_field(const Field& field)
 
 void DbAddDel::treeviewcolumn_on_cell_data(Gtk::CellRenderer* renderer, const Gtk::TreeModel::iterator& iter, int model_column_index)
 {
-  const guint col_real = model_column_index + get_count_hidden_system_columns();
-  Gtk::TreeModel::Row treerow = *iter;
-  Gnome::Gda::Value value;
-  treerow->get_value(col_real, value);
-
-  DbAddDelColumnInfo& column_info = m_ColumnTypes[model_column_index];
-  switch(column_info.m_field.m_field.get_glom_type())
+  if(iter)
   {
-    case(Field::TYPE_BOOLEAN):
-    {
-      Gtk::CellRendererToggle* pDerived = dynamic_cast<Gtk::CellRendererToggle*>(renderer);
-      pDerived->set_active( value.get_bool() ); 
+    const guint col_real = model_column_index + get_count_hidden_system_columns();
+    Gtk::TreeModel::Row treerow = *iter;
+    Gnome::Gda::Value value;
+    treerow->get_value(col_real, value);
 
-      break;
+    DbAddDelColumnInfo& column_info = m_ColumnTypes[model_column_index];
+    switch(column_info.m_field.m_field.get_glom_type())
+    {
+      case(Field::TYPE_BOOLEAN):
+      {
+        Gtk::CellRendererToggle* pDerived = dynamic_cast<Gtk::CellRendererToggle*>(renderer);
+        pDerived->set_active( value.get_bool() ); 
+
+        break;
+      }
+      default:
+      {
+        //TODO: Maybe we should have custom cellrenderers for time, date, and numbers.
+        Gtk::CellRendererText* pDerived = dynamic_cast<Gtk::CellRendererText*>(renderer);
+
+        const Glib::ustring text = GlomConversions::get_text_for_gda_value(column_info.m_field.m_field.get_glom_type(), value, column_info.m_field.m_numeric_format);
+        //g_assert(text != "NULL");
+        pDerived->property_text() = text;
+
+        break;
+      } 
     }
-    default:
-    {
-      //TODO: Maybe we should have custom cellrenderers for time, date, and numbers.
-      Gtk::CellRendererText* pDerived = dynamic_cast<Gtk::CellRendererText*>(renderer);
-
-      pDerived->property_text() = GlomConversions::get_text_for_gda_value(column_info.m_field.m_field.get_glom_type(), value);
-
-      break;
-    } 
   }
 }
 
@@ -1599,9 +1715,14 @@ App_Glom* DbAddDel::get_application()
 {
   Gtk::Container* pWindow = get_toplevel();
   //TODO: This only works when the child widget is already in its parent.
-  
+
   return dynamic_cast<App_Glom*>(pWindow);
 }
-  
-      
+
+void DbAddDel::set_allow_view(bool val)
+{
+  m_allow_view = val;
+}
+
+
 
