@@ -25,6 +25,7 @@
 #include "../../data_structure/layout/layoutitem_field.h"
 #include <libgdamm.h>
 #include "glom_db_treemodel.h"
+#include "../../document/document_glom.h"
 
 #include <vector>
 #include <map>
@@ -51,7 +52,9 @@ class DbTreeViewColumnGlom;
 
 //For adding/deleting/selecting multi-columned lists of items.
 //This was also an abstraction layer against the strangeness of GtkSheet, though it now uses Gtk::TreeView instead.
-class DbAddDel : public Gtk::VBox
+class DbAddDel
+ : public Gtk::VBox,
+   public View_Composite_Glom
 {
 public:
   friend class InnerIgnore; //declared below.
@@ -127,11 +130,22 @@ public:
 
   virtual Field get_key_field() const;
   virtual void set_key_field(const Field& field);
- 
+
+  virtual void set_table_name(const Glib::ustring& table_name);
+
+  ///For instance, if the user can not view the table then don't try to get the records.
+  virtual void set_allow_view(bool val = true);
+
   /** @result The index of the new column.
    */
   virtual guint add_column(const LayoutItem_Field& field);
 
+  /// Specify which records to show:
+  virtual void set_where_clause(const Glib::ustring& where_clause);
+
+  /// Start using the added columns.
+  virtual void set_columns_ready();
+
   virtual guint get_columns_count() const;
 
   virtual LayoutItem_Field get_column_field(guint column_index) const;
@@ -173,8 +187,9 @@ public:
   virtual void set_rules_hint(bool val = true);
 
   //Signals:
-  //row number.
-  typedef sigc::signal<void, const Gtk::TreeModel::iterator&> type_signal_user_added;
+
+  //row number, col number.
+  typedef sigc::signal<void, const Gtk::TreeModel::iterator&, guint> type_signal_user_added;
   type_signal_user_added signal_user_added();
 
   //row number, col number.
@@ -285,6 +300,7 @@ protected:
   //Columns, not including the hidden internal columns:
   typedef std::vector<DbAddDelColumnInfo> type_ColumnTypes;
   type_ColumnTypes m_ColumnTypes;
+  Glib::ustring m_table_name, m_where_clause;
 
   Gtk::Menu* m_pMenuPopup;
   Glib::RefPtr<Gtk::ActionGroup> m_refActionGroup;
@@ -304,6 +320,9 @@ protected:
 
   Field m_key_field;
 
+  bool m_columns_ready;
+  bool m_allow_view;
+
   //signals:
   type_signal_user_added m_signal_user_added;
   type_signal_user_changed m_signal_user_changed;
